"{ Package: 'stx:goodies/xtreams/core' }"

"{ NameSpace: Xtreams }"

RingBuffer subclass:#ElasticBuffer
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'Xtreams-Core'
!

ElasticBuffer comment:'ElasticBuffer is a variation on a ring buffer that will grow if it has no room to write the data in to its cache. It will split the cache and insert new elements if necessary, otherwise it will wrap around like a regular ring buffer. You have a slightly more liberal ''rewind'' policy than the RingBuffer such that if you read data, then write data, it will only overwrite the read data if the write data can fit in to that space, otherwise it will grow the cache.

'
!


!ElasticBuffer methodsFor:'accessing - writing'!

put: anObject

	self writeSize < 1 ifTrue: [self growBy: 1].
	^super put: anObject
!

write: anInteger from: aSequenceableCollection at: startIndex

	anInteger <= self writeSize ifFalse: [self growBy: anInteger - self writeSize].
	self privateWrite: anInteger from: aSequenceableCollection at: startIndex.
	^anInteger
! !

!ElasticBuffer methodsFor:'private'!

clearCache
	cache := cache copyEmpty: 0
!

growBy: anInteger
	| split replacement growth |
	split := dataPosition + writePosition.
	cache size isZero ifFalse: [split := split \\ cache size].
	split isZero ifTrue: [split := cache size].
	growth := cache size + anInteger + anInteger.
	replacement := cache copyEmpty: cache size + growth.
	replacement replaceFrom: 1 to: split with: cache startingAt: 1.
	replacement replaceFrom: split + growth + 1 to: replacement size with: cache startingAt: split + 1.
	cache := replacement.
	split <= dataPosition ifTrue: [dataPosition := dataPosition + growth]
! !

!ElasticBuffer methodsFor:'testing'!

hasFixedWriteSpace
	^false
!

hasSpaceToWrite
	^true
! !

!ElasticBuffer class methodsFor:'documentation'!

version_SVN
    ^ '$Id$'
! !
