"{ Package: 'stx:goodies/xtreams/transforms' }"

"{ NameSpace: Xtreams }"

WriteStream subclass:#ObjectWriteStream
	instanceVariableNames:'marshaler objects nothing int16 int32 int64 uint16 uint32 uint64
		float double int8'
	classVariableNames:''
	poolDictionaries:''
	category:'Xtreams-Transforms'
!

ObjectWriteStream comment:'Marshals objects into a binary destination using the associated ObjectMarshaler. The marshaler version ID is emited at the beginning.

Instance Variables
	marshaler	<ObjectMarshaler> performs marshaling of object bodies
	objects	<Array> retains unmarshaled objects for resolution of backward references (to preserve identity)
	longs	<InterpretedReadStream> (internal) for reading of longs
	longlongs	<InterpretedReadStream> (internal) for reading of longlongs
	floats	<InterpretedReadStream> (internal) for reading of floats
	doubles	<InterpretedReadStream> (internal) for reading of doubles
	nothing	<Object> (internal) an object used to occupy empty slots in objects (since nil is potentially valid content)

'
!


!ObjectWriteStream class methodsFor:'instance creation'!

on: aDestination
	^self on: aDestination marshaler: ObjectMarshaler new
!

on: aDestination marshaler: aMarshaler
	^self new on: aDestination marshaler: aMarshaler
! !

!ObjectWriteStream methodsFor:'accessing'!

put: object
	marshaler marshal: self object: object.
	self complete.
	^object
!

write: anInteger from: aSequenceableCollection at: startIndex
	startIndex to: anInteger + startIndex - 1 do: [:index |
		marshaler marshal: self object: (aSequenceableCollection at: index).
		self complete].
	^anInteger
! !

!ObjectWriteStream methodsFor:'initialize-release'!

contentsSpecies
	^Array
!

on: aDestination marshaler: aMarshaler
	super on: aDestination.
	marshaler := aMarshaler.
	objects := Array with: Transcript.
	nothing := Object new.

	int8 := aDestination interpreting: #signedChar.
	(marshaler configureMarshal: aDestination)
		ifTrue: [
			int16 := aDestination interpreting: #signedShort_be.
			int32 := aDestination interpreting: #signedLong_be.
			int64 := aDestination interpreting: #signedLonglong_be.
			uint16 := aDestination interpreting: #unsignedShort_be.
			uint32 := aDestination interpreting: #unsignedLong_be.
			uint64 := aDestination interpreting: #unsignedLonglong_be]
		ifFalse: [
			int16 := aDestination interpreting: #signedShort_le.
			int32 := aDestination interpreting: #signedLong_le.
			int64 := aDestination interpreting: #signedLonglong_le.
			uint16 := aDestination interpreting: #unsignedShort_le.
			uint32 := aDestination interpreting: #unsignedLong_le.
			uint64 := aDestination interpreting: #unsignedLonglong_le].
	float := aDestination interpreting: #float.
	double := aDestination interpreting: #double
! !

!ObjectWriteStream methodsFor:'primitives'!

double
	^double
!

float
	^float
!

int16
	^int16
!

int32
	^int32
!

int64
	^int64
!

int8
	^int8
!

uint16
	^uint16
!

uint32
	^uint32
!

uint64
	^uint64
!

uint8
	^destination
! !

!ObjectWriteStream methodsFor:'private'!

complete
	objects atAllPut: nothing.
	objects at: 1 put: Transcript
!

grow
	objects := objects copyWith: nothing
!

nothing
	^nothing
!

objects
	^objects
! !

!ObjectWriteStream class methodsFor:'documentation'!

version_SVN
    ^ '$Id$'
! !
